﻿using C1.Win.C1FlexGrid;
using System;
using System.Windows.Forms;

namespace FlexGridSelectionModeTest
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            #region FlexGrid setup

            const int rowCount = 10;
            const int columnCount = 4;

            this.c1FlexGrid1.AllowFiltering = true;
            this.c1FlexGrid1.AutoClipboard = true;

            this.c1FlexGrid1.Rows.Count = rowCount;
            this.c1FlexGrid1.Cols.Count = columnCount;
            this.c1FlexGrid1.Cols[1].Name = "ID";
            this.c1FlexGrid1.Cols[2].Name = "Name";
            this.c1FlexGrid1.Cols[3].Name = "Description";

            this.c1FlexGrid1.Cols[1].DataType = typeof(int);
            this.c1FlexGrid1.Cols[2].DataType = typeof(string);
            this.c1FlexGrid1.Cols[3].DataType = typeof(string);

            this.c1FlexGrid1.Cols[1].Caption = this.c1FlexGrid1.Cols[1].Name;
            this.c1FlexGrid1.Cols[2].Caption = this.c1FlexGrid1.Cols[2].Name;
            this.c1FlexGrid1.Cols[3].Caption = this.c1FlexGrid1.Cols[3].Name;


            for (int i = 1; i < rowCount; i++)
            {
                this.c1FlexGrid1.Rows[i].UserData = i + 1;
                this.c1FlexGrid1[i, 1] = i + 1;
                this.c1FlexGrid1[i, 2] = "Name" + (i + 1).ToString();
                this.c1FlexGrid1[i, 3] = i % 2;
            }

            #endregion
        }

        private void btnRowSelection_Click(object sender, EventArgs e)
        {
            //To get the single cell, you can use the C1FlexGrid.Row and C1FlexGrid.Col properties
            if (c1FlexGrid1.Selection.IsSingleCell)
            {
                MessageBox.Show($"Cell: [{c1FlexGrid1.Row}, {c1FlexGrid1.Col}]");
                return;
            }

            //To get the Selected Rows, you can use the c1FlexGrid1.Rows.Selected property,
            //instead of using the C1FlexGrid.Selection property
            string rows = string.Empty;
            for (int i = 0; i < c1FlexGrid1.Rows.Selected.Count; i++)
            {
                rows += $"{c1FlexGrid1.Rows.Selected[i]["ID"]}{Environment.NewLine}";
            }
            MessageBox.Show(rows);
        }
    }

    class FlexGridEx : C1FlexGrid
    {
        public FlexGridEx()
        {
            //Set the SelectionMode to ListBox
            base.SelectionMode = SelectionModeEnum.ListBox;
        }

        //Cannot change because implementation is specific to ListBox SelectionMode
        public new SelectionModeEnum SelectionMode
        {
            get { return base.SelectionMode; }
        }

        public override bool IsCellSelected(int row, int col)
        {
            //If a single cell is selected and only 1 row is there in Rows.Selected (SelectionModeEnum.ListBox)
            if (Selection.IsSingleCell && Rows.Selected.Count <= 1)
            {
                //Only single cell should be selected
                if (row == Row && col == Col)
                    return true;
                else
                    return false;
            }

            //else call base class method to do default processing
            return base.IsCellSelected(row, col);
        }

        protected override void OnAfterSelChange(RangeEventArgs e)
        {
            base.OnAfterSelChange(e);

            //After multiple rows as selected, the behavior of IsCellSelected method changes
            //and you will need to invalidate the first seleted row
            Invalidate(Row, Cols.Fixed - 1, Row, Cols.Count - 1);
        }

        protected override void OnPreviewKeyDown(PreviewKeyDownEventArgs e)
        {
            base.OnPreviewKeyDown(e);

            //Handle Ctrl+C press on a single cell manually
            if (e.Control && e.KeyCode == Keys.C)
            {
                if (Selection.IsSingleCell)
                {
                    AutoClipboard = false;
                    Clipboard.SetText(GetDataDisplay(Row, Col));
                }
                else
                {
                    AutoClipboard = true;
                }
            }

            //Handle Ctrl+V press on a single cell manually
            if(e.Control && e.KeyCode == Keys.V && !AutoClipboard)
            {
                if(Selection.IsSingleCell)
                {
                    SetData(Row, Col, Clipboard.GetText(), true);
                }
                else
                {
                    AutoClipboard = true;
                }
            }
        }
    }
}
